/* 
 * Copyright (c) 2008,2013
 *     IS/AIST-ST2I/CNRS Joint Japanese-French Robotics Laboratory (JRL).
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdlib.h>
#include <err.h>
#include <assert.h>

#include "viam_private.h"

/* --- local data ---------------------------------------------------- */

static void *	swap[4];	/**< temporary buffer for image filtering */
static size_t	swapsize[4];	/**< size of swap buffer */
static size_t	totalsize;


/* --- viam_swap ----------------------------------------------------- */

/** \brief Get swap buffer
 *
 * \return A pointer on the swap buffer.
 */

void *
viam_swap(unsigned int bank)
{
  assert(bank < sizeof(swap)/sizeof(swap[0]));
  return swap[bank];
}


/* --- viam_setswap -------------------------------------------------- */

/** \brief Allocate swap buffer
 *
 * \param[in] bank	Swap area number.
 * \param[in] size	Required amount of swap space.
 *
 * \return A pointer on the swap buffer.
 */

void
viam_setswap(unsigned int bank, size_t size)
{
  assert(bank < sizeof(swap)/sizeof(swap[0]));

  if (size == 0) {
    if (swap[bank]) free(swap[bank]);
    totalsize -= swapsize[bank];
    swap[bank] = NULL;
    swapsize[bank] = 0;
    return;
  }
  if (size <= swapsize[bank]) return;

  if (swap[bank]) free(swap[bank]);
  swap[bank] = malloc(size);
  totalsize -= swapsize[bank];
  swapsize[bank] = size;
  if (!swap[bank]) {
    warnx("cannot allocate %d bytes for camera swap", (unsigned int)size);
    swapsize[bank] = 0;
  }
  totalsize += swapsize[bank];

  warnx("%d bytes of swap space allocated for image filtering",
        (unsigned int)totalsize);
}
